;  9960 extender version 2 - Trying motion ignore
;
; Use John Fine's S3C8ASM program to build it as follows:
; S3C8ASM 9960ex2.ASM -A9960ex2.HEX -L9960ex2.LST
;
; Read the associate ReadMe.txt file for instructions.
;
; The numbered comments in the code, like ;1; refer to more detailed explanations
; at the bottom of this file.

OldKeyMoveArea = 24	
NewKeyMoveArea = 401	;1;
UpgradeArea    = 100
LearnedArea    = 400
EepromSize     = 800	;2K
O_DevIndex     = 21

;2; Signature	NAKxNAK2	
	xorg	2
	org	2
	db	4E, 41, 4B, 78, 4E, 41, 4B, 32

;3; Original key move and macro area

	xorg	OldKeyMoveArea
	org	OldKeyMoveArea

	;Key move to load extender
	db	3A	;Format
	db	23	;0->2*(dev index 1 = TV) ;3->body length
	dw	1708	;1 = TV ;$708 = 1800
	db	0	;hex cmd
	db	0	;Mark end of original KeyMove area

;4; Mark end of learned signal area
	xorg	LearnedArea
	org	LearnedArea
	db	00

;5; New key move code and macro area

	xorg	NewKeyMoveArea
	org	NewKeyMoveArea

	;Define key codes
	Shift	= 02	;Setup
	TV     	= 01
	CBL    	= 11
	VCR    	= 0A
	DVD    	= 21
	CD_    	= 09	; _ used because "CD" is a hex number.
	AUDIO  	= 22
	AUX    	= 23
	PVR    	= 26
	HT     	= 12
	SCROLL 	= 18
	
	;Define phantom codes for device selection
	;Leading underscore on DevIndexes (to distinguish from dev keys)
	;Trailing underscore on Key sets

	_CBL	= 47
	_TV	= 4E
	_VCR	= 55
	_DVD	= 5C
	_CD	= 63
	_PVR	= 6A
	_AUDIO	= 71
	_AUX	= 78
	
	O_	= 6	; Other
	P_	= 5	; Pip
	M_	= 4	; Menu
	C_	= 3	; Channel
	V_	= 2	; Volume
	T_	= 1	; Transport
	X_	= 0	; Temporary device selection
	
	X_CANCEL=7F

;6;  Define initial macros to make the device keys work
	db	CBL,    16,  M_+_CBL,	 C_+_CBL,    V_+_CBL,	T_+_CBL,  P_+_CBL, O_+_CBL
	db	HT,     16,  M_+_CBL,    C_+_CBL,    V_+_CBL,  	T_+_CBL,  P_+_CBL, O_+_CBL
	db	TV,     16,  M_+_TV,     C_+_TV,     V_+_TV,   	T_+_TV,   P_+_TV,  O_+_TV
	db	VCR,    15,  M_+_VCR,    C_+_VCR,           	T_+_VCR,  P_+_VCR, O_+_VCR
	db	DVD,    15,  M_+_DVD,    C_+_DVD,             	T_+_DVD,  P_+_DVD, O_+_DVD
	db	CD_,    15,  M_+_CD,     C_+_CD,              	T_+_CD,   P_+_CD,  O_+_CD
	db	PVR,    15,  M_+_PVR,    C_+_PVR,             	T_+_PVR,  P_+_PVR, O_+_PVR
	db	AUX,    15,  M_+_AUX,    C_+_AUX,   		T_+_AUX,  P_+_AUX, O_+_AUX
	db	AUDIO,  14,  M_+_AUDIO,  C_+_AUDIO,  V_+_AUDIO,   		   O_+_AUDIO

	db	00	;Mark end of key moves and macros codes

;7;   ROM Calls and registers
Ram		=	FF00
Read255		=	3470	;Read 225 bytes EEPROM[WWA] -> FF00
				;Leaves WE=255, WWC=FFFF, RR40=FF00
Read		=	0648	;Read WE bytes EEPROM[WWA] -> @WWC
				;Preserves WE, advances WWC
BlinkW2Times	=	3185	;Blink LED W2 times  			
Blink2		=	3177	;Blink LED twice - only used for Debugging
DelayWW0	=	010D	;Delay and do nothing
ScanKeypad	=	300B	;Scan keypad
ScrollScr	=	2D58	;Scroll screen based on R_Scr
SelScrHT	=	07AF	;Select HT screen
SelScrNorm	=	07D3	;Select screen based on O_Dev
LoadScreenData	=	0314	;Load screen control data and validation 
				;keylist for type of R_DevA
LookupKeyList	=	2C97	;Lookup R_Key in keylist (@WW2)

BacklightOn	=	02DF	;Turn on backlight
SelectTimeout	=	3941	;Select a timeout
HandleTimeOut	=	398C	;Handle timeout
StartT0		=	35DF	;Start Timer0
WaitNoKeys	=	3066	;Wait until no keys are pressed
DebounceKeys	=	2FED	;Debounced scan of keypad
ReadHeader	=	0BDA	;Read header of KeyMove, Macro, or FavList
PopWbWa		=	06BD	;pop RCB ;pop RCA ;ret
LoadKeyMove	=	0D38	;Load a key move
DoKeyMove	=	2B4F	;Do a KeyMove
DoNormalKey	=	3840	;Do a Normal Key
ValidateKey	=	2CE5	;Check if R_Key is illuminated, based on 
				;type of R_DevA and R_Scr
RemapKey	=	2D2D	;Convert R_Key based on type of R_DevA and 
				;R_Scr (i.e. PLAY,PAUSE -> PAGE+,-)

R_SetDev	=	RBF	;Used to remember last device button pressed.
R_Scr		=	R6A	;Current screen number 
R_SFlags	=	R4C	;(X)Shift flags
R_Dev		=	R68	;Current DevIndex
R_DevA		=	R69	;Active DevIndex
R_Key1		=	R6C	;Debounced key
R_Closures	=	R6F	;Count of keys simultaneously down
R_Key		=	R6E	;Key being processed
R_FirstPress	=	R64	;First key press toggle
R_Flags		=	R71	;flags
R_IFlags	=	R73	;Interupt flags
R_Mac		=	R9B	;Macro pointer
AR_MacBuf	=	9C	;Start of macro buffer
AR_MacLim	=	B6	;Limit of macro buffer
R_MacLast	=	RB5	;Last byte of macro buffer
R_DevTbl	=	RB7	;Table of DevIndexes
R_DevX		=	RB7	;Temporary DevIndex
AR_DevT		=	B8	;Address of DevIndex for "Transport" keys
AR_DevV		=	B9	;Address of DevIndex for "Volume" keys
AR_DevC		=	BA	;Address of DevIndex for "Channel" keys
AR_DevM		=	BB	;Address of DevIndex for "Menu" keys
AR_DevP		=	BC	;Address of DevIndex for "Pip" keys
R_DevO		=	RBD	;DevIndex for "Other" keys
R_FavDev	=	RBE	;DevIndex for Fav Key
R_FavState	=	R75	;Position within Fav list
R_ToadTog	=	R65	;Status register for ToadTog
R_BklightTimer	=	R67	;Address of Backlight timer $023 in RDF
W_Key		=	RC7
R_Motion        =       R63     ;Motion detector test

;8; Upgrade area
	xorg	UpgradeArea
	org	UpgradeArea
	dw	D_table
	dw	P_table

	DT_index = 0
	P_addr  = D_addr

;---------------------------------------------------------
; ---- Device/Protocol upgrades for loading extender
;---------------------------------------------------------
tv_1800:
	db	80	;Protocol = 0x180
	db	0	;No digit map
	db	1	;No defined keys
	
					D_addr = $$
					xorg	P_addr
					org	$$
Protocol_180:
	db	00,00,01
					org	Ram

	LD	W2, 04		;Blink LED 4 times
	CALL	BlinkW2Times	
	CLR	R4C		
	
 	LDW	RCA, HT_patch 	;Address of Fav/HT Dev settings in eeprom
 	LDW	RCC, AR_DevT
 	LD	WE, 7	
 	CALL	Read

	LDW	RCA, ee_main	;Address of extender in eeprom
	LDW	RDA, Start	;Return to Start after Read255
	ENTER	Read255		;Read 225 bytes EEPROM[WWA] -> FF00

					P_addr = $$
					xorg	D_table+2+DT_index+DT_index
					org	$$
	dw	1F08		;Type=TV, Protocol>255, setup code = 0x708 = 1800
					xorg	D_table2+DT_index+DT_index
					org	$$
	dw	tv_1800
					xorg	P_table+2+DT_index+DT_index
					org	$$
	dw	180		;Protocol ID
					xorg	P_table2+DT_index+DT_index
					org	$$
	dw	Protocol_180
					DT_index = DT_index+1

;---------------------------------------------------------
; ---- Device/Protocol upgrades for ToadTog
;---------------------------------------------------------
					xorg	D_addr
					org	$$
VCR_1800:
	db	81	;Protocol = 0x181
	db	0	;No digit map
	db	1	;No defined keys
	
					D_addr = $$
					xorg	P_addr
					org	$$
Protocol_181:
	db	00,00,01
					org	Ram
	LD	W2, R03		;Get the "HEX command" from the KeyMove
	SWAP	RC2		;Top digit selects bit number
	LD	W1, 1		;Start with bit 0
L181_1:	AND	RC2, 7		;Use low three bits as bit number, also handle
				; overflow when rotating 8 times for bit 0
	RL	RC1		;Rotate bit into position
	DJNZ	W2, L181_1	;Do it N time for 1 to 7, and 8 times for 0
	LD	W2, R03		;Get the "HEX command" from the KeyMove
	AND	R03, 7		;Length of TRUE side
	LD	W3, R3A		;Body length of keymove
	SUB	RC3, 3		;keylist length
	TM	RC1, R_ToadTog	;Is the bit set?
	JREQ	L181_3		;No
	LD	W3, R03		;Length of TRUE side
	CLR	R03
L181_3:	CP	RC3, R03	;Selected side empty?
	JRLE	L181_4		;Yes
	LD	W4, R2D[W3]	;Get a key code
	CP	R_Mac, AR_MacBuf;Buffer full?
	JRULE	L181_4		;Yes
	PUSHUD	@R_Mac, RC4	;Insert KeyCode into pending keys
	DJNZ	W3, L181_3
	
L181_4:	BTJRF	L181_5, W2.3	;Set requested?
	OR	R_ToadTog, RC1	;Yes: do it
L181_5:	BTJRT	L181_6, W2.7	;Toggle requested?
	XOR	R_ToadTog, RC1	;Yes: do it
L181_6:	RET
					P_addr = $$
					xorg	D_table+2+DT_index+DT_index
					org	$$
	dw	2F08		;Type=VCR, Protocol>255, setup code = 0x708 = 1800
					xorg	D_table2+DT_index+DT_index
					org	$$
	dw	VCR_1800
					xorg	P_table+2+DT_index+DT_index
					org	$$
	dw	181		;Protocol ID
					xorg	P_table2+DT_index+DT_index
					org	$$
	dw	Protocol_181
					DT_index = DT_index+1

;---------------------------------------------------------
; ---- Device/Protocol upgrades for Device Specific Macro
;---------------------------------------------------------
					xorg	D_addr
					org	$$
tv_1103:
	db	FC	;Protocol = 0x1FC
	db	0	;No digit map
	db	1	;No defined keys
	
					D_addr = $$
					xorg	P_addr
					org	$$
Protocol_1FC:
	db	00,00,01
					org	Ram
	LD	W3, R3A		;Body length of keymove
	DJNZ	W3, L1FC_2	;Subtract 2 byte SetupCode before starting
L1FC_1:	CP	R_Mac, AR_MacBuf
	JRULE	L1FC_3		;Pending buffer full
	LD	W4, R2C[W3]	;Get a key code
	PUSHUD	@R_Mac, RC4	;Insert KeyCode into pending keys
L1FC_2:	DJNZ	W3, L1FC_1
L1FC_3:	RET
					P_addr = $$
					xorg	D_table+2+DT_index+DT_index
					org	$$
	dw	1C4F		;Type=TV, Protocol>255, setup code = 0x44F = 1103
					xorg	D_table2+DT_index+DT_index
					org	$$
	dw	tv_1103
					xorg	P_table+2+DT_index+DT_index
					org	$$
	dw	1FC		;Protocol ID
					xorg	P_table2+DT_index+DT_index
					org	$$
	dw	Protocol_1FC
					DT_index = DT_index+1

;---------------------------------------------------------
; ---- Device/Protocol upgrades for pause
;---------------------------------------------------------
					xorg	D_addr
					org	$$
tv_1104:
	db	FB	;Protocol = 0x1FB
	db	0	;No digit map
	db	1	;No defined keys
	
					D_addr = $$
					xorg	P_addr
					org	$$
Protocol_1FB:
	db	00,00,01
					org	Ram
	LD	RC0, R03	;Get count
	LD	RC1, R03
	CALL	DelayWW0	;Pause by using Delay
	RET
					P_addr = $$
					xorg	D_table+2+DT_index+DT_index
					org	$$
	dw	1C50		;Type=TV, Protocol>255, setup code = 0x450 = 1104
					xorg	D_table2+DT_index+DT_index
					org	$$
	dw	tv_1104
					xorg	P_table+2+DT_index+DT_index
					org	$$
	dw	1FB		;Protocol ID
					xorg	P_table2+DT_index+DT_index
					org	$$
	dw	Protocol_1FB
					DT_index = DT_index+1

;---------------------------------------------------------
; ---- Device/Protocol upgrades for Long/Double Key Press
;---------------------------------------------------------
					xorg	D_addr
					org	$$
TV_1106:
	db	F9	;Protocol = 0x1F9
	db	0	;No digit map
	db	1	;No defined keys
	
					D_addr = $$
					xorg	P_addr
					org	$$
Protocol_1F9:
	db	00,00,01
					org	Ram
	LD	R4A, R03	;Get the "HEX command" from the KeyMove
	SWAP	R4A		;Timeout value in high nibble
	AND	R4A, 0F		;Set timeout
	CALL	StartT0

	AND	R03, 07		;Length of SHORT/SINGLE press side
	LD	W3, R3A		;Body length of keymove
	SUB	RC3, 3		;keylist length

	TM	R2D, 08		;Check for a Double Key Press requested?
	JRNZ	L1F9_7

	; Check for Long Key Press
L1F9_1:	CALL	ScanKeypad
	JRNC	L1F9_5		;Key is no longer pressed
L1F9_2:	TM	R_IFlags, 20	;Test timeout
	JRZ	L1F9_1
	JR	L1F9_3

	; Check for Double Key Press
L1F9_7:	LDW	RC0, 2500	;Change this value to adjust sensitivity
	CALL	DelayWW0
	AND	R_IFlags, 7F	;Clear key interrupt test bit
	LD	RF1, FF		;Enable key sense interrupts
L1F9_6:	IDLE
	TM	R_IFlags, 80	;Test key interrupt
	JRNZ	L1F9_3
	TM	R_IFlags, 20	;Test timeout
	JRZ	L1F9_6

L1F9_5:	LD	W3, R03		;Length of SHORT/SINGLE press side
	CLR	R03
L1F9_3:	CP	RC3, R03	;Selected side empty?
	JRLE	L1F9_4		;Yes
	LD	W4, R2D[W3]	;Get a key code
	CP	R_Mac, AR_MacBuf;Buffer full?
	JRULE	L1F9_4		;Yes
	PUSHUD	@R_Mac, RC4	;Insert KeyCode into pending keys
	DJNZ	W3, L1F9_3
L1F9_4:	RET

					P_addr = $$
					xorg	D_table+2+DT_index+DT_index
					org	$$
	dw	1C52		;Type=TV, Protocol>255, setup code = 0x452 = 1106
					xorg	D_table2+DT_index+DT_index
					org	$$
	dw	TV_1106
					xorg	P_table+2+DT_index+DT_index
					org	$$
	dw	1F9		;Protocol ID
					xorg	P_table2+DT_index+DT_index
					org	$$
	dw	Protocol_1F9
					DT_index = DT_index+1

;---------------------------------------------------------
; ---- Tables
;---------------------------------------------------------
					DT_Size = DT_index
					xorg	P_addr
					org	$$
D_table:
	dw	DT_Size		;Number of devices
	D_table2 = D_table+2+DT_Size+DT_Size
	P_table  = D_table2+DT_Size+DT_Size
					xorg	P_table
					org	$$
	dw	DT_Size		;Number of protocols
	P_table2 = P_table+2+DT_Size+DT_Size
					xorg	P_table+DT_Size+DT_Size

;===========================================================================
;---- Extender - MAIN   ----  Top of EEPROM 
;===========================================================================

	org	Ram
	xorg	ee_main		;9;

Start:	;Initialization only needed one time.
	CLR	RD9		;Init stack pointer

Main:	LD	R_DevX, WC	;Turn off temporary DevIndex

L10:	CALL	WaitNoKeys	;Wait for no keys pressed
	DI
	LD	RF1, FF		;Enable button sense interrupts
	CLR	RF2		;Clear pending interrupts (buttons)

L18:	CALL	SelectTimeout	;Select timeout value
	JRNC	L40		;No Timeout needed

L20:	LD	R4A, R_BklightTimer	;Set timeout
	CLR	R_IFlags     		;Clear interrupt flags
	CALL	StartT0			;Start timer0

L30:	IDLE
	TM	R_IFlags, B0	;Is a button pressed or motion detected?
	JRLT	L50		;Yes
	JREQ	L30		;No interrupt flag set

 	CALL	HandleTimeOut	;Turn off backlight

L40:	OR	RE5, 04 	;Enable motion detector interrupts
	LD	RFB, A5		;Enable stop
	EI
	STOP
	NOP
	NOP
	DI
 	TM	R_IFlags, 90	;Test motion detector and key interrupt flags
 	JRZ	L40		;Sleep again if neither interrupt occured

	CP      R_IFlags,  80  	;Needs to test if it was motion or button
	JRNC     L45

	CP	R_Motion, 01 	;1 equals USE
	JRNE	L40


L45:  	OR	R_Flags, 81 	;Set backlight enable, Flag backlight on
 	CALL	BacklightOn
	CLR	RE5		;Disable motion detector interrupts
	CLR	RE6		;Clear pending interrupts (motion detector)
	CP	R_FirstPress, 01 ;1 equals USE
	JRNE	L10

L50:	CALL	DebounceKeys	;Keypad scan
	DEC	R_Closures	;Exactly one key pressed?
	JRNE	L10		;No

	LD	R_Key, R_Key1
  	CALL	ValidateKey
	JRNC	L10			;Skip non-illuminated key

  	CALL	RemapKey		;Convert R_Key if necessary

 	LD	R_Mac, AR_MacLim-1  	;Init macro pointer
 	LD	R_MacLast, R_Key    	;Key code
 	OR	R_MacLast, R_SFlags 	;Shift the key if flag is set
 	CLR	R_SFlags	    		;Clear shift flag
 	CP	R_MacLast, Shift    	;Is this the shift key?
; 078D    -------------;
Shift_Key_patch = $$-1
;----------------------;
 	JREQ	L55		 ;Yes
	CP	R_MacLast, Shift ;Alternate shift?
; 0792   --------------;
XShift_Key_patch = $$-1
;----------------------;
	JRNE	L60		;No
	LD	R_SFlags, C0	;Yes: set xshift flag
; 0797   ---------------;
xshift_bits_patch = $$-1
;----------------------;
L55:	OR	R_SFlags, 80	;Yes: set shift flag

L60:	LDW	RDA, L200	;Go to L200 in Part 2 after Read255

L99:	LDW	RCA, ee_part2	;EEPROM address of part2
	ENTER	Read255		;Read 255 bytes from eeprom to ram

ChgScr:	CALL	LoadScreenData
	CP	R_Key, SCROLL
	JPNE	ChgDv1
	CALL	ScrollScr		;Scroll 1 screen
	JP	ChgDv3		;Scroll done

ChgDv1:	CP	R_SetDev, 01	;Is it HT
	JPNE	ChgDv2		;No
	CALL	SelScrHT		;Selects HT screen
	JP	ChgDv3		;HT Change done

ChgDv2:	AND	R72,#DF
	CALL	SelScrNorm	;Selects screen based on O_Device button	

ChgDv3:	LDW	RDA, ScrDone	
	JR	L99		;Return to ScrDone in part 2

HtDevKey:	;Lookup R_key in HT key sets and return DevIndex in W1
 	LDW	RCA, HTKeyLists
 	LD	W1, R_DevO	;Set O_ device as default
L83: 	LDCI	W0, @WA		;W0 <- AR_Dev?
 	BTJRF	L85, W0.3	;20;Branch if no more keylist left
 	LDCI	W2, @WA		;WW2 <- keylist address
 	LDCI	W3, @WA
 	CALL	LookupKeyList
 	JRNC	L83
 	LD	W1, @W0		;W1 <- R_Dev?
L85:	LDW	RDA, L206	;Return to L206 in ee_part2
	JR	L99

T_KeyList	=	2EE6	;[FFWD,Rewind,Stop,pause,play,record,skipfwd{pip-move},skipback{pip-freeze},liveTV{PIP_Main}]
V_KeyList	=	2DEB	;[vol+/-,mute]
C_KeyList	=	2EF4	;[digits,CH+/-,enter{skip},input(TV/VCR),Fav,last]
M_KeyList	=	2F05	;[up,down,left,right,OK_surround,info,exit,menu,guide,page+{Audio},page-{subtitle}]
P_KeyList	=	2EE6	;[skipback{pip-freeze},skipfwd{pip-move},liveTV{PIP_Main},Format_replay]
O_KeyList	=	2DBC	;[Macro1-4,Setup,Fav,HT2,Scroll,HT,PVR-VOD{preset},Device buttons,Phantoms]

HTKeyLists:
	DB	AR_DevM
	DW	M_KeyList
	DB	AR_DevT
	DW	T_KeyList
	DB	AR_DevP
	DW	P_KeyList
	DB	AR_DevV
	DW	V_KeyList
	DB	AR_DevC
	DW	C_KeyList
	
	DB	00

	Must_Be_00FF_Or_Less = $-Ram
; 07F9  ---------------;
HT_patch	=	$$
;----------------------;
 	db	00,03,00,00,00,00  ;Initial HT devices (00=CBL)
 	db	00		   ;DevIndex for Fav Key (FF=none)

	Must_Be_00FF_Or_Less = $-Ram

ee_main = ee_main + EepromSize - $$	;7;

;===========================================================================
;---- Extender part 2 - Lower EEPROM
;===========================================================================

	org	Ram
	xorg	ee_part2	;Must be greater than 0029

L201:	LDW	RRC2,Read255	;Read 225 bytes EEPROM[WWA] -> FF00
	PUSH	RC3
 	PUSH	RC2
	LDW	RRC2,ee_part2	;EEPROM address of extender
	PUSH	RC2
	PUSH	RC3
	LDW	RRC2,PopWbWa	;pop RCB / pop RCA / ret
	PUSH	RC3
	PUSH	RC2
	LD	R_Key, W_Key	;Get the KeyCode
 	AND	R_Key, 3F	;Ignoring shift and xshift
 	LD	W1, R_DevX	;Temporary DevIndex
 	BTJRF	L210, W1.3	;Branch if temporary DevIndex valid
	 			;Lookup R_key in HT key sets and return DevIndex in W1
	LDW	RDA, HtDevKey	;Return to HtDevKey in ee_main
	JR	L299

L206:	;<=== Returns here from HtDevKey

L210:	LD	R_DevA, W1	;Set Active DevIndex
	LDW	RCA, NewKeyMoveArea-FF	;EEProm address to search

L230:	ADD	WB, WE		;Advance to next entry
	ADC	RCA, 0

L235:	CALL	ReadHeader	;Read header of KeyMove, Macro, or FavList

L239:	TM	W0, W0		;End of list?
	JPEQ	DoNormalKey	;Yes
	SUB	W0, W_Key	;Is it the right key?
	JRNE	L230		;No
	BTJRT	L250, W1.4	;Branch if it's a macro or FavList
	DIV	RC0, 20		;W1 <- device ;W0 <- body length
	CP	RC1, R_DevA	;Is it the right device?
	JRNE	L230		;No
	LD	R3A, W0		;Save length for variable length KeyMoves
	LDW	RRDA,DoKeyMove
	ENTER	LoadKeyMove

L270:	LD	R_Closures, W0	;If 1, ROM will hold while pressed
	POPUI	W_Key, @R_Mac	;Get next key from macro
	TM	W_Key, W_Key	;Is it 00?
	JREQ	L240		;Yes: Skip it

	LD	W1, W_Key	;Make a copy
	SUB	RC1, _CBL	;Is it a command to change a DevIndex?
	JRGE	L280		;Yes

	CP	R_Key, HT	;See if it's the HT button
	JPNE	L275		;No

	LD	R_SetDev, 01	;Set HT flag for ChgScr	

L275:	CP	R_Key, SCROLL	;Is it scroll command to change screen?
	JPNE	L201		;No: Normal processing
	JP	L285

L280:	CLR	RC0		;Prepare for DIV
	DIV	RC0, 7
	LD	R_DevTbl[W0],W1
	CP	RC0, O_		;Change screen if R_DevO changes
	JRNE	L200

	LD	R_Dev, W1	;22;

L285:	LDW	RDA, ChgScr	;Jump to ChgScr in ee_main
	JR	L299

ScrDone:;<=== Returns here from ChgScr

L200:	LD	RD9, FE		;Adjust stack to repush L200 return address

L240:	LD	W0, AR_MacLim	;Limit of macro buffer
 	SUB	RC0, R_Mac	;Number of keys left in buffer
 	JRUGT	L270		;Branch if there are keys left

	CLR	R_SetDev

	LDW	RDA, Main	;Return to Main in first part
L299:	LDW	RCA, ee_main	;EEPROM address of first part
	ENTER	Read255

L250:	;Found a macro or FavList ;W0 = 0 W1 = 2'nd byte of header
				  ;WWA = Address of macro header 
				  ;WE = Body length in bytes plus 2

 	SUB	RCE, 2		;Length
L251:	AND	R_FavState, 0F
 	LD	WC, R_FavState
 	MULT	RCC, 5		;WD <- FavState * 5 ;WC <- 0
 	BTJRF	L255, W1.5	;Branch if it's not a FavList
 	INC	R_FavState
 	CP	WD, WE
 	JRGE	L251
 	ADD	WB, WD
 	ADC	WA, WC
 	LD	WE, 5
 	LD	R_DevX, R_FavDev
L255:	LD	WD, R_Mac	;Get previous macro pointer
 	SUB	WD, WE		;Compute new start
 	CP	RCD,AR_MacBuf	;Would it fit?
 	JPULT	L239		;Nesting error (too big)

	INCW	RCA
	INCW	RCA
	LD	R_Mac, WD
 	JP	Read		;Read in the new macro

	Must_Be_00FF_Or_Less = $-Ram

ee_part2 = ee_part2 + 100 - $$	;7; +100	;Must be greater than 0029   

; Notes referred to by number in the code above
;
; <All need to be rewritten>
;20; This bit test works because all AR_Dev? have the bit 3 set and the
;    terminator ($00) doesn't.
;22; This is necessary for the Device Multiplexer/Toggler protocols to work.
